from django.test import TestCase
from rest_framework.reverse import reverse
from ..models import Author, Post
from django.contrib.auth.models import User
from django.contrib.comments import Comment
from django.contrib.contenttypes.models import ContentType
from django.utils.timezone import now
import json


class ViewTests(TestCase):

    def setUp(self):
        #make a bunch of objects for the tests
        self.test_password = password = 'test'
        test_admin = User.objects.create_superuser('test', 'test@codegur.us',
                                                   password)
        self.test_normal = User.objects.create_user('test2', 'test@codegur.us', password)
        test_admin.first_name = "Test"
        test_admin.last_name = "Tester"
        test_admin.save()

        self.client.login(username=test_admin.username, password=password)

        post = Post.objects.create(
            author=test_admin,
            title="test post title",
            body="test post body",
            slug="test-post-title"
        )

    def test_api_root(self):
        resp = self.client.get(reverse('api_root'))
        self.assertEqual(resp.status_code, 200)

    def test_api_posts_list(self):
        resp = self.client.get(reverse('posts-list'))
        self.assertEqual(resp.status_code, 200)

    def test_api_post_detail(self):
        resp = self.client.get(reverse('post-detail', kwargs={'pk': 1}))
        self.assertEqual(resp.status_code, 200)

    def test_api_authors_list(self):
        resp = self.client.get(reverse('authors-list'))
        self.assertEqual(resp.status_code, 200)

    def test_api_author_detail(self):
        resp = self.client.get(reverse('author-detail', kwargs={'pk': 1}))
        self.assertEqual(resp.status_code, 200)
    
    def test_api_create_post(self):
        author_url = reverse('author-detail', kwargs={'pk': 1})
        data = {
            'author': author_url,
            'title': 'second test post',
            'slug': 'who-cares',
            'body': '2nd test post body',
            'published': True,
        }
        url = reverse('posts-list')
        #make a post with a dictionary via restore_object
        resp = self.client.post(url,
                               data=json.dumps(data),
                               content_type='application/json'
                               )
        
        self.assertEqual(resp.status_code, 201)

    def test_api_update_post(self):
        author_url = reverse('author-detail', kwargs={'pk': 1})
        data = {
            'author': author_url,
            'title': 'test post renamed',
            'slug': 'who-cares',
            'body': 'test post body2',
            'published': True,
        }
        url = reverse('post-detail', kwargs={'pk': 1})
        resp = self.client.put(url,
                               data=json.dumps(data),
                               content_type='application/json'
                               )
        
        self.assertEqual(resp.status_code, 200)

    #ensure read only for user that isn't post author
    def test_api_update_post_not_author(self):
        author_url = reverse('author-detail', kwargs={'pk': 2})
        data = {
            'author': author_url,
            'title': 'test post renamed',
            'slug': 'who-cares',
            'body': 'test post body2',
            'published': True,
        }
        url = reverse('post-detail', kwargs={'pk': 1})

        self.client.login(username=self.test_normal, password=self.test_password)
        resp = self.client.put(url,
                               data=json.dumps(data),
                               content_type='application/json'
                               )
        
        self.assertEqual(resp.status_code, 403)

    def test_all_archive(self):
        resp = self.client.get(reverse('all_archive'))
        self.assertEqual(resp.status_code, 200)

    def test_yearly_archive(self):
        year = now().year
        resp = self.client.get(reverse('yearly_archive',
                                       kwargs={'year': year})
                               )
        self.assertEqual(resp.status_code, 200)

    def test_monthly_archive(self):
        year = now().year
        month = str(now().month)
        # month has to be a 2 digit int represented as a string
        if len(month) == 1:
            month = "".join(['0', month])
        resp = self.client.get(reverse('monthly_archive',
                                       kwargs={'year': year, 'month': month})
                               )
        self.assertEqual(resp.status_code, 200)

    def test_tag_archive(self):
        tag_string = "testtag"
        resp = self.client.get(reverse('tag_archive',
                                       kwargs={'tag': tag_string})
                               )
        self.assertEqual(resp.status_code, 200)

    def test_author_archive(self):
        author_string = "Testy-Testington"
        resp = self.client.get(reverse('author_archive',
                                       kwargs={'author': author_string})
                               )
        self.assertEqual(resp.status_code, 200)

    #ensure that when a comment is created that
    #the page redirects to the post commented on
    def test_comment_posted(self):
        comment = Comment.objects.create(
            id=1,
            object_pk=1,
            site_id=1,
            content_type=ContentType.objects.get(model='post')
        )
        resp = self.client.get(reverse('comment_posted'),
                               data={'c': comment.id}
                               )
        self.assertEqual(resp.status_code, 302)

        resp = self.client.get(reverse('comment_posted'),
                               data={'c': 1},
                               follow=True
                               )
        self.assertEqual(resp.status_code, 200)

    def test_view_post(self):
        slug = "test-post-title"
        resp = self.client.get(reverse('view_post', kwargs={'slug': slug}))
        self.assertEqual(resp.status_code, 200)

    def test_rss_feed_renders(self):
        resp = self.client.get(reverse('latest_entries_rss'))
        self.assertEqual(resp.status_code, 200)



from django.test import TestCase
from rest_framework.reverse import reverse


class ContextProcessorTest(TestCase):

    #make sure that the dictionary is added to the request
    def test_context(self):
        resp = self.client.get(reverse('all_archive'))
        self.assertIn('BLOG_TITLE', resp.context)



from django.db import models
from django.core.urlresolvers import reverse
from django.contrib.auth.models import User
from django.conf import settings
from django.utils.translation import ugettext_lazy as _
from taggit.managers import TaggableManager
from .managers import PostManager

class ClassName(object):
    """docstring for ClassName"""
    def __init__(self, arg):
        super(ClassName, self).__init__()
        self.arg = arg


BLOG_SETTINGS = settings.BLOG_SETTINGS['defaults']


class Person(models.Model):
    first_name = models.CharField(max_length=30)

    # class Meta:
    #     proxy = True

    def get_absolute_url(self):
        return super(Person, self).get_absolute_url()

    def __unicode__(self):
        return super(Person, self).__unicode__()

class Author(User):
    """A Proxy for User. Overrides some getters"""

    class Meta:
        proxy=True

    def get_absolute_url(self):
        return reverse('author_archive',
                       args=['-'.join([self.first_name, self.last_name])])

    def __unicode__(self):
        return ' '.join([self.first_name, self.last_name])


class Post(models.Model):
    """
    User generated blog post
    """

    author = models.ForeignKey(User, blank=True, verbose_name=_("author"))
    title = models.CharField(max_length=200, unique=True,
                             verbose_name=_("title"))
    body = models.TextField(verbose_name=_("body"))
    created_at = models.DateTimeField(auto_now_add=True,
                                      verbose_name=_("created at"))
    # TODO: last modified?
    published = models.BooleanField(default=BLOG_SETTINGS['auto_publish'],
                                    verbose_name=_("published?"))
    promoted = models.BooleanField(default=BLOG_SETTINGS['auto_promote'],
                                    verbose_name=_("promoted?"))
    tags = TaggableManager() #models.ManyToManyField(Tag, blank=True, verbose_name=_("tags"))
    slug = models.SlugField(max_length=200, unique=True,
                            verbose_name=_("slug"))

    objects = models.Manager()  # The default manager.
    popular_posts = PostManager()

    class Meta:
        verbose_name = _("post")
        verbose_name_plural = _("posts")

    def __unicode__(self):
        return self.title

    def get_absolute_url(self):
        return reverse('view_post', args=[str(self.slug)])

    def get_tags(self):
        """Returns tags as a composite string"""
        names = ', '.join([t.name for t in self.tags.all()])
        #if len(names) > 20:
        #    names = names[:20] + "..."
        return names
    get_tags.short_description = _("Tags")



from django.test import TestCase
from ..models import Author, Post

from django.contrib.auth.models import User


class ModelTests(TestCase):

    def setUp(self):
        #make a bunch of objects for the tests
        password = 'test'
        test_admin = User.objects.create_superuser('test', 'test@test.com',
                                                   password)
        test_admin.first_name = "Test"
        test_admin.last_name = "Tester"
        test_admin.save()

        self.client.login(username=test_admin.username, password=password)

        Post.objects.create(
            author=test_admin,
            title="test post title",
            body="test post body",
            slug="test-post-title"
        )

    def test_post_new(self):
        author = User.objects.get(pk=1)
        p = Post()
        p.author=author
        p.title="test post title2"
        p.body="test post body"
        p.slug="test-post-title2"
        p.save()
        self.assertEqual(p, Post.objects.get(slug=p.slug))
        #self.assertEqual(p.published, BLOG_SETTINGS['defaults']['auto_publish'])
        #self.assertEqual(p.promoted, BLOG_SETTINGS['defaults']['auto_promote'])
        

    def test_author_url(self):
        author = Author.objects.get(pk=1)
        resp = self.client.get(author.get_absolute_url())
        self.assertEqual(resp.status_code, 200)

    def test_post_unicode(self):
        post = Post.objects.get(pk=1)
        self.assertEqual(post.title, str(post))

    def test_post_get_tags(self):
        post = Post.objects.get(pk=1)
        post.tags.set('a,b,c')
        self.assertEqual(post.get_tags(), 'a,b,c')

    def test_author_create(self):
        author = Author.objects.create_user('author_test', 'author_test@test.come', 'test')
        self.assertEqual(author, Author.objects.get(username='author_test'))



from django.test import TestCase
from ..models import Author, Post
from django.contrib.auth.models import User


class AdminTests(TestCase):

    def setUp(self):
        #make a bunch of objects for the tests
        password = 'test'
        test_admin = User.objects.create_superuser('test', 'test@codegur.us',
                                                   password)
        test_admin.first_name = "Test"
        test_admin.last_name = "Tester"
        test_admin.save()

        self.client.login(username=test_admin.username, password=password)

    def test_entry_add_and_change(self):
        """Test the insertion of a Post"""
        
        post_data = {'title': u'New entry',
                     'body': u'BODY',
                     'slug': u'new-entry',
                     'tags': u'test',
                    }

        response = self.client.post('/admin/blogger/post/add/', post_data, follow=True)
        self.assertEquals(response.status_code, 200)
        self.assertEquals(Post.objects.count()>0, True)

        post = Post.objects.get(title="New entry")
        
        self.assertEquals(post.author, User.objects.get(pk=1))



from django.test import TestCase
from ..models import Author, Post
from django.utils.translation import ugettext_lazy as _
from django.contrib.auth.models import User
from ..feeds import LatestEntriesFeed

#test the feed https://github.com/Fantomas42/django-blog-zinnia/blob/master/zinnia/tests/feeds.py#L141
class TestFeeds(TestCase):

    def setUp(self):
        #make a bunch of objects for the tests
        password = 'test'
        test_admin = User.objects.create_superuser('test', 'test@test.com',
                                                   password)
        test_admin.first_name = "Test"
        test_admin.last_name = "Tester"
        test_admin.save()

        self.client.login(username=test_admin.username, password=password)

        Post.objects.create(
            author=test_admin,
            title="test post title",
            body="test post body",
            slug="test-post-title"
        )

    def test_latest_entries(self):
        
        feed = LatestEntriesFeed()
        self.assertEquals(feed.link, '/rss/')
        self.assertEquals(len(feed.items()), 1)
        self.assertEquals(feed.title, _("Blog RSS"))
        self.assertEquals(
            feed.description,
            _('Updates on changes and additions to this blog.'))


from django.test import TestCase
from ..models import Post
from django.contrib.auth.models import User

class ManagerTests(TestCase):

    def setUp(self):
        password = 'test'
        test_admin = User.objects.create_superuser('test', 'test@test.com',
                                                   password)
        test_admin.first_name = "Test"
        test_admin.last_name = "Tester"
        test_admin.save()

        self.client.login(username=test_admin.username, password=password)

        Post.objects.create(
            author=test_admin,
            title="test post title",
            body="test post body",
            slug="test-post-title"
        )

    #test that the comment count is present on the queryset
    def test_comment_count(self):
        post = Post.popular_posts.get(pk=1)
        self.assertTrue(hasattr(post, 'comment_count'))
